@@ -12,10 +12,13 @@ extern "C" {
 #include <sys/time.h>
 #include <limits.h>
 #include <time.h>
+#include <signal.h>
+#ifndef S_SPLINT_S
+#include <pthread.h>	/* pacifies OpenBSD's compiler */
+#endif
 
-#define MAXTAGLEN	6	/* maximum length of sentence tag name */
-#define MAXCHANNELS	12	/* maximum GPS channels (*not* satellites!) */
-#define NMEA_CHANNELS	12	/* max channels allowed in NMEA format */
+#define MAXTAGLEN	8	/* maximum length of sentence tag name */
+#define MAXCHANNELS	14	/* maximum GPS channels (*not* satellites!) */
 #define SIRF_CHANNELS	12	/* max channels allowed in SiRF format */
 
 /* 
@@ -42,8 +45,7 @@ extern "C" {
  * timestamps and some data in this structure can be up to 1 cycle (usually
  * 1 second) older than the fix time.
  *
- * Time uncertainty is expected value (mean).
- * Position and velocity uncertainties are 1-sigma values with a mean of zero.
+ * Error estimates are at 95% confidence.
  */
 struct gps_fix_t {
     double time;	/* Time of update, seconds since Unix epoch */
@@ -64,6 +66,103 @@ struct gps_fix_t {
     double eps;		/* Speed uncertainty, meters/sec */
     double climb;       /* Vertical speed, meters/sec */
     double epc;		/* Vertical speed uncertainty */
+    double pitch;       /* Pitch angle in degrees */
+    double roll;        /* Roll angle in degrees */
+    double dip;         /* Dip angle in degrees */
+};
+
+/*  
+ * From the RCTM104 standard:
+ *
+ * "The 30 bit words (as opposed to 32 bit words) coupled with a 50 Hz
+ * transmission rate provides a convenient timing capability where the
+ * times of word boundaries are a rational multiple of 0.6 seconds."
+ *
+ * "Each frame is N+2 words long, where N is the number of message data
+ * words. For example, a filler message (type 6 or 34) with no message
+ * data will have N=0, and will consist only of two header words. The
+ * maximum number of data words allowed by the format is 31, so that
+ * the longest possible message will have a total of 33 words."
+ */
+#define RTCM_WORDS_MAX	33
+#define MAXCORRECTIONS	18	/* max correction count in type 1 or 9 */
+#define MAXSTATIONS	10	/* maximum stations in almanac, type 5 */
+/* RTCM104 doesn't specify this, so give it the largest reasonable value */
+#define MAXHEALTH	(RTCM_WORDS_MAX-2)	
+
+/*
+ * A nominally 30-bit word (24 bits of data, 6 bits of parity)
+ * used both in the GPS downlink protocol described in IS-GPS-200
+ * and in the format for DGPS corrections used in RTCM-104.
+ */
+typedef /*@unsignedintegraltype@*/ unsigned int isgps30bits_t;
+
+struct rtcm_t {
+    /* header contents */
+    unsigned type;	/* RTCM message type */
+    unsigned length;	/* length (words) */
+    double   zcount;	/* time within hour: GPS time, no leap secs */
+    unsigned refstaid;	/* reference station ID */
+    unsigned seqnum;	/* nessage sequence number (modulo 8) */
+    unsigned stathlth;	/* station health */
+
+    /* message data in decoded form */
+    union {
+	struct {
+	    unsigned int nentries;
+	    struct rangesat_t {		/* data from messages 1 & 9 */
+		unsigned ident;		/* satellite ID */
+		unsigned udre;		/* user diff. range error */
+		unsigned issuedata;	/* issue of data */
+		double rangerr;		/* range error */
+		double rangerate;	/* range error rate */
+	    } sat[MAXCORRECTIONS];
+	} ranges;
+	struct {		/* data for type 3 messages */
+	    bool valid;		/* is message well-formed? */
+	    double x, y, z;
+	} ecef;
+	struct {		/* data from type 4 messages */
+	    bool valid;		/* is message well-formed? */
+	    enum {gps, glonass, unknown} system;
+	    enum {local, global, invalid} sense;
+	    char datum[6];
+	    double dx, dy, dz;
+	} reference;
+	struct {		/* data from type 5 messages */
+	    unsigned int nentries;
+	    struct consat_t {
+		unsigned ident;		/* satellite ID */
+		bool iodl;		/* issue of data */
+		unsigned int health;	/* is satellite healthy? */
+#define HEALTH_NORMAL		(0)	/* Radiobeacon operation normal */
+#define HEALTH_UNMONITORED	(1)	/* No integrity monitor operating */
+#define HEALTH_NOINFO		(2)	/* No information available */
+#define HEALTH_DONOTUSE		(3)	/* Do not use this radiobeacon */
+	       int snr;			/* signal-to-noise ratio, dB */
+#define SNR_BAD	-1			/* not reported */
+		unsigned int health_en;	/* health enabled */
+		bool new_data;		/* new data? */
+		bool los_warning;	/* line-of-sight warning */
+		unsigned int tou;	/* time to unhealth, seconds */
+	    } sat[MAXHEALTH];
+	} conhealth;
+	struct {		/* data from type 7 messages */
+	    unsigned int nentries;
+	    struct station_t {
+		double latitude, longitude;	/* location */
+		unsigned int range;		/* range in km */
+		double frequency;		/* broadcast freq */
+		unsigned int health;		/* station health */
+		unsigned int station_id;	/* of the transmitter */
+		unsigned int bitrate;		/* of station transmissions */
+	    } station[MAXSTATIONS];
+	} almanac;
+	/* data from type 16 messages */
+	char message[(RTCM_WORDS_MAX-2) * sizeof(isgps30bits_t)];
+	/* data from messages of unknown type */
+	isgps30bits_t	words[RTCM_WORDS_MAX-2];
+    } msg_data;
 };
 
 typedef /*@unsignedintegraltype@*/ unsigned int gps_mask_t;
@@ -90,14 +189,17 @@ struct gps_data_t {
 #define VERR_SET	0x00010000u
 #define PERR_SET	0x00020000u
 #define SATELLITE_SET	0x00040000u
-#define SPEEDERR_SET	0x00080000u
-#define TRACKERR_SET	0x00100000u
-#define CLIMBERR_SET	0x00200000u
-#define DEVICE_SET	0x00400000u
-#define DEVICELIST_SET	0x00800000u
-#define DEVICEID_SET	0x01000000u
-#define ERROR_SET	0x02000000u
-#define CYCLE_START_SET	0x04000000u
+#define PSEUDORANGE_SET	0x00080000u
+#define USED_SET	0x00100000u
+#define SPEEDERR_SET	0x00200000u
+#define TRACKERR_SET	0x00400000u
+#define CLIMBERR_SET	0x00800000u
+#define DEVICE_SET	0x01000000u
+#define DEVICELIST_SET	0x02000000u
+#define DEVICEID_SET	0x04000000u
+#define ERROR_SET	0x08000000u
+#define CYCLE_START_SET	0x10000000u
+#define RTCM_SET	0x20000000u
 #define FIX_SET		(TIME_SET|MODE_SET|TIMERR_SET|LATLON_SET|HERR_SET|ALTITUDE_SET|VERR_SET|TRACK_SET|TRACKERR_SET|SPEED_SET|SPEEDERR_SET|CLIMB_SET|CLIMBERR_SET)
     double online;		/* NZ if GPS is on line, 0 if not.
 				 *
@@ -126,7 +228,7 @@ struct gps_data_t {
     double pdop, hdop, vdop, tdop, gdop;	/* Dilution of precision */
 
     /* redundant with the estimate elments in the fix structure */
-    double epe;  /* estimated spherical position error, 1 sigma (meters)  */
+    double epe;  /* spherical position error, 95% confidence (meters)  */
 
     /* satellite status -- valid when satellites > 0 */
     int satellites;		/* # of satellites in view */
@@ -134,13 +236,36 @@ struct gps_data_t {
     int elevation[MAXCHANNELS];	/* elevation of satellite */
     int azimuth[MAXCHANNELS];	/* azimuth */
     int ss[MAXCHANNELS];	/* signal strength */
-    int part, await;		/* for tracking GSV parts */
+
+#if 0	/* not yet used or filled in */
+    /* measurement data */
+    double pseudorange[MAXCHANNELS];	/* meters */
+    double deltarange[MAXCHANNELS];	/* meters/sec */
+    double doppler[MAXCHANNELS];	/* Hz */
+    unsigned satstat[MAXCHANNELS];	/* tracking status */
+#define SAT_ACQUIRED	0x01		/* satellite acquired */
+#define SAT_CODE_TRACK	0x02		/* code-tracking loop acquired */
+#define SAT_CARR_TRACK	0x04		/* carrier-tracking loop acquired */
+#define SAT_DATA_SYNC	0x08		/* data-bit synchronization done */
+#define SAT_FRAME_SYNC	0x10		/* frame synchronization done */
+#define SAT_EPHEMERIS	0x20		/* ephemeris collected */
+#define SAT_FIX_USED	0x40		/* used for position fix */
+#endif
+
+    /* compass status */
+    char headingStatus;
+    char pitchStatus;
+    char rollStatus;
+    double horzField;   /* Magnitude of horizontal magnetic field */
 
     /* where and what gpsd thinks the device is */
     char	gps_device[PATH_MAX];	/* only valid if non-null. */
     char	*gps_id;	/* only valid if non-null. */
     unsigned int baudrate, parity, stopbits;	/* RS232 link parameters */
     unsigned int driver_mode;	/* whether driver is in native mode or not */
+
+    /* RTCM-104 data */
+    struct rtcm_t	rtcm;
     
     /* device list */
     int ndevices;		/* count of available devices */
@@ -158,23 +283,12 @@ struct gps_data_t {
     double emit_time;		/* emission time (-> E2) */
     double c_recv_time;		/* client receipt time (-> T2) */
     double c_decode_time;	/* client end-of-decode time (-> D2) */
-    unsigned int cycle;		/* refresh cycle time in seconds */
+    double cycle, mincycle;	/* refresh cycle time in seconds */
 
     /* these members are private */
     int gps_fd;			/* socket or file descriptor to GPS */
     void (*raw_hook)(struct gps_data_t *, char *, size_t len, int level);/* Raw-mode hook for GPS data. */
     void (*thread_hook)(struct gps_data_t *, char *, size_t len, int level);/* Thread-callback hook for GPS data. */
-    int seen_sentences;		/* track which sentences have been seen */
-#define GPRMC	0x01
-#define GPGGA	0x02
-#define GPGLL	0x04
-#define GPVTG	0x08
-#define GPGSA	0x10
-#define GPGSV	0x20
-#define GPZDA	0x40
-#define PGRME	0x80
-    struct tm nmea_date;
-    double subseconds;
 };
 
 extern struct gps_data_t *gps_open(const char *host, const char *port);
@@ -182,9 +296,14 @@ int gps_close(struct gps_data_t *);
 int gps_query(struct gps_data_t *gpsdata, const char *requests);
 int gps_poll(struct gps_data_t *gpsdata);
 void gps_set_raw_hook(struct gps_data_t *gpsdata, void (*hook)(struct gps_data_t *sentence, char *buf, size_t len, int level));
-    int gps_set_callback(struct gps_data_t *gpsdata, void (*callback)(struct gps_data_t *sentence, char *buf, size_t len, int level), pthread_t *handler);
+int gps_set_callback(struct gps_data_t *gpsdata, void (*callback)(struct gps_data_t *sentence, char *buf, size_t len, int level), pthread_t *handler);
 int gps_del_callback(struct gps_data_t *gpsdata, pthread_t *handler);
 
+enum unit {unspecified, imperial, nautical, metric};
+enum unit gpsd_units(void);
+enum deg_str_type { deg_dd, deg_ddmm, deg_ddmmss };
+extern /*@observer@*/ char *deg_to_str( enum deg_str_type type,  double f);
+
 extern void gps_clear_fix(/*@ out @*/struct gps_fix_t *);
 extern void gps_merge_fix(/*@ out @*/struct gps_fix_t *, 
 			  gps_mask_t,
@@ -195,9 +314,9 @@ extern double timestamp(void);
 extern double iso8601_to_unix(char *);
 extern /*@observer@*/char *unix_to_iso8601(double t, /*@ out @*/char[], int len);
 extern double gpstime_to_unix(int, double);
+extern void unix_to_gpstime(double, /*@out@*/int *, /*@out@*/double *);
 extern double earth_distance(double, double, double, double);
-extern double wgs84_separation(double lat, double lon);
-extern int gpsd_units(void);
+extern double wgs84_separation(double, double);
 
 /* some multipliers for interpreting GPS output */
 #define METERS_TO_FEET	3.2808399	/* Meters to U.S./British feet */
@@ -211,9 +330,9 @@ extern int gpsd_units(void);
 /* miles and knots are both the international standard versions of the units */
 
 /* angle conversion multipliers */
-#define PI 3.1415926535897932384626433832795029L
-#define RAD_2_DEG  57.2957795130823208767981548141051703
-#define DEG_2_RAD  0.0174532925199432957692369076848861271
+#define PI      	3.1415926535897932384626433832795029
+#define RAD_2_DEG	57.2957795130823208767981548141051703
+#define DEG_2_RAD	0.0174532925199432957692369076848861271
 
 /* gps_open() errno return values */
 #define NL_NOSERVICE	-1	/* can't get service entry */
