@@ -280,21 +280,21 @@ static int filesock(char *filename)
 static struct gps_device_t channels[MAXDEVICES];
 #define allocated_channel(chp)	((chp)->gpsdata.gps_device[0] != '\0')
 #define free_channel(chp)	(chp)->gpsdata.gps_device[0] = '\0'
+#define syncing(chp)	(chp->gpsdata.gps_fd>-1&& chp->packet_type==BAD_PACKET)
 
 static struct subscriber_t {
     double active;			/* when subscriber signed on */
     bool tied;				/* client set device with F */
     bool watcher;			/* is client in watcher mode? */
     int raw;				/* is client in raw mode? */
-#ifdef RTCM104_SERVICE
-    bool rtcm;				/* is RTCM what he actually wants? */
-#endif /* RTCM104_SERVICE */
+    enum {GPS,RTCM104,ANY} requires;	/* type of device requested */
     /*@relnull@*/struct gps_device_t *device;	/* device subscriber listens to */
 } subscribers[FD_SETSIZE];		/* indexed by client file descriptor */
 
 static void detach_client(int cfd)
 {
     (void)close(cfd);
+    gpsd_report(4, "detaching %d in detach_client\n", cfd);
     FD_CLR(cfd, &all_fds);
     subscribers[cfd].raw = 0;
     subscribers[cfd].watcher = false;
@@ -324,11 +324,11 @@ static ssize_t throttled_write(int cfd, char *buf, ssize_t len)
     if ((status = write(cfd, buf, (size_t)len)) > -1)
 	return status;
     if (errno == EBADF)
-	gpsd_report(3, "Client on %d has vanished.\n", cfd);
+	gpsd_report(3, "client(%d) has vanished.\n", cfd);
     else if (errno == EWOULDBLOCK && timestamp() - subscribers[cfd].active > NOREAD_TIMEOUT)
-	gpsd_report(3, "Client on %d timed out.\n", cfd);
+	gpsd_report(3, "client(%d) timed out.\n", cfd);
     else
-	gpsd_report(3, "Client write to %d: %s\n", cfd, strerror(errno));
+	gpsd_report(3, "client(%d) write: %s\n", cfd, strerror(errno));
     detach_client(cfd);
     return status;
 }
@@ -376,8 +376,7 @@ static /*@null@*/ /*@observer@*/struct gps_device_t *find_device(char *device_na
     return NULL;
 }
 
-static /*@null@*/ struct gps_device_t *open_device(char *device_name, 
-						   bool nowait)
+static /*@null@*/ struct gps_device_t *open_device(char *device_name)
 /* open and initialize a new channel block */
 {
     struct gps_device_t *chp;
@@ -391,17 +390,41 @@ static /*@null@*/ struct gps_device_t *open_device(char *device_name,
 found:
     gpsd_init(chp, &context, device_name);
     chp->gpsdata.raw_hook = raw_hook;
-    if (nowait) {
-	if (gpsd_activate(chp) < 0) {
-	    return NULL;
-	}
-	FD_SET(chp->gpsdata.gps_fd, &all_fds);
+    if (gpsd_activate(chp) < 0) {
+	return NULL;
     }
-
+    gpsd_report(4, "flagging descriptor %d in open_device\n", chp->gpsdata.gps_fd);
+    FD_SET(chp->gpsdata.gps_fd, &all_fds);
     return chp;
 }
 /*@ +globstate @*/
 
+static bool allocation_policy(struct gps_device_t *channel,
+			      struct subscriber_t *user,
+			      double most_recent)
+{
+#ifdef __UNUSED__
+    /* only allocate devices that we know the packet type of */
+    if (channel->packet_type == BAD_PACKET)
+	return false;
+#endif /* __UNUSED__ */
+    /* maybe we have already bound a more recently active device */
+    if (user->device!=NULL && channel->gpsdata.sentence_time < most_recent)
+	return false;
+    gpsd_report(1, "User requires %d, channel type is %d\n", user->requires, channel->packet_type);
+    /* we might have type constraints */
+    if (user->requires == ANY)
+	return true;
+    else if (user->requires==RTCM104 && (channel->packet_type==RTCM_PACKET))
+	return true;
+    else if (user->requires == GPS 
+	     && (channel->packet_type!=RTCM_PACKET) && (channel->packet_type!=BAD_PACKET))
+	return true;
+    else
+	return false;
+    return true;
+}
+
 /*@ -branchstate -usedef -globstate @*/
 static bool assign_channel(struct subscriber_t *user)
 {
@@ -410,14 +433,11 @@ static bool assign_channel(struct subscriber_t *user)
 	double most_recent = 0;
 	struct gps_device_t *channel;
 
+	gpsd_report(4, "client(%d): assigning channel...\n", user-subscribers);
 	/* ...connect him to the most recently active device */
 	for(channel = channels; channel<channels+MAXDEVICES; channel++)
 	    if (allocated_channel(channel)) {
-		if ((user->device == NULL || channel->gpsdata.sentence_time >= most_recent)
-#ifdef RTCM104_SERVICE
-		    && (user->rtcm == (channel->packet_type == RTCM_PACKET))
-#endif /* RTCM104_SERVICE */
-		    ) {
+		if (allocation_policy(channel, user, most_recent)) {
 		    user->device = channel;
 		    most_recent = channel->gpsdata.sentence_time;
 		}
@@ -425,16 +445,22 @@ static bool assign_channel(struct subscriber_t *user)
     }
 
     if (user->device == NULL) {
-	gpsd_report(1, "channel assignment failed.\n");
+	gpsd_report(1, "client(%d): channel assignment failed.\n", user-subscribers);
 	return false;
     }
 
     /* and open that device */
-    if (user->device->gpsdata.gps_fd == -1) {
+    if (user->device->gpsdata.gps_fd != -1) 
+	gpsd_report(1,"client(%d): channel %d already active.\n",
+		    user-subscribers, user->device->gpsdata.gps_fd);
+    else {
 	gpsd_deactivate(user->device);
-	if (gpsd_activate(user->device) < 0) 
+	if (gpsd_activate(user->device) < 0) {
+	    
+	    gpsd_report(1, "client(%d): channel activation failed.\n", user-subscribers);
 	    return false;
-	else {
+	} else {
+	    gpsd_report(4, "flagging descriptor %d in assign_channel\n", user->device->gpsdata.gps_fd);
 	    FD_SET(user->device->gpsdata.gps_fd, &all_fds);
 	    if (user->watcher && !user->tied) {
 		(void)write(user-subscribers, "F=", 2);
@@ -608,6 +634,25 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 	    else
 		(void)strcpy(phrase, ",F=?");
 	    break;
+	case 'G':
+	    if (*p == '=') {
+		gpsd_report(1,"<= client(%d): requesting data type %s\n",cfd,++p);
+		if (strncasecmp(p, "rtcm104", 7) == 0)
+		    whoami->requires = RTCM104;
+		else if (strncasecmp(p, "gps", 3) == 0)
+		    whoami->requires = GPS;
+		else
+		    whoami->requires = ANY;
+		p += strcspn(p, ",\r\n");
+	    }
+	    (void)assign_channel(whoami);
+	    if (whoami->device==NULL||whoami->device->packet_type==BAD_PACKET)
+		(void)strcpy(phrase, ",G=?");
+	    else if (whoami->device->packet_type == RTCM_PACKET)
+		(void)snprintf(phrase, sizeof(phrase), ",G=RTCM104");
+	    else
+		(void)snprintf(phrase, sizeof(phrase), ",G=GPS");
+	    break;
 	case 'I':
 	    if (assign_channel(whoami) && whoami->device->device_type!=NULL)
 		(void)snprintf(phrase, sizeof(phrase), ",I=%s", 
@@ -629,7 +674,7 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 	    phrase[strlen(phrase)-1] = '\0';
 	    break;
 	case 'L':
-	    (void)snprintf(phrase, sizeof(phrase), ",L=2 " VERSION " abcdefiklmnopqrstuvwxyz");	//ghj
+	    (void)snprintf(phrase, sizeof(phrase), ",L=2 " VERSION " abcdefgiklmnopqrstuvwxyz");	//hj
 	    break;
 	case 'M':
 	    if (!assign_channel(whoami) && (!whoami->device || whoami->device->gpsdata.fix.mode == MODE_NOT_SEEN))
@@ -755,47 +800,49 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 		(void)strcpy(phrase, ",P=?");
 	    break;
 	case 'Q':
+#define ZEROIZE(x)	(isnan(x)!=0 ? 0.0 : x)  
 	    if (assign_channel(whoami) && 
 		(isnan(whoami->device->gpsdata.pdop)==0
 		 || isnan(whoami->device->gpsdata.hdop)==0
 		 || isnan(whoami->device->gpsdata.vdop)==0))
 		(void)snprintf(phrase, sizeof(phrase), ",Q=%d %.2f %.2f %.2f %.2f %.2f",
 			whoami->device->gpsdata.satellites_used, 
-			whoami->device->gpsdata.pdop, 
-			whoami->device->gpsdata.hdop, 
-			whoami->device->gpsdata.vdop,
-			whoami->device->gpsdata.tdop,
-			whoami->device->gpsdata.gdop);
+			ZEROIZE(whoami->device->gpsdata.pdop), 
+			ZEROIZE(whoami->device->gpsdata.hdop), 
+			ZEROIZE(whoami->device->gpsdata.vdop),
+			ZEROIZE(whoami->device->gpsdata.tdop),
+			ZEROIZE(whoami->device->gpsdata.gdop));
 	    else
 		(void)strcpy(phrase, ",Q=?");
+#undef ZEROIZE
 	    break;
 	case 'R':
 	    if (*p == '=') ++p;
 	    if (*p == '2') {
 		(void)assign_channel(whoami);
 		subscribers[cfd].raw = 2;
-		gpsd_report(3, "%d turned on super-raw mode\n", cfd);
+		gpsd_report(3, "client(%d) turned on super-raw mode\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",R=2");
 		p++;
 	    } else if (*p == '1' || *p == '+') {
 		(void)assign_channel(whoami);
 		subscribers[cfd].raw = 1;
-		gpsd_report(3, "%d turned on raw mode\n", cfd);
+		gpsd_report(3, "client(%d) turned on raw mode\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",R=1");
 		p++;
 	    } else if (*p == '0' || *p == '-') {
 		subscribers[cfd].raw = 0;
-		gpsd_report(3, "%d turned off raw mode\n", cfd);
+		gpsd_report(3, "client(%d) turned off raw mode\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",R=0");
 		p++;
 	    } else if (subscribers[cfd].raw) {
 		subscribers[cfd].raw = 0;
-		gpsd_report(3, "%d turned off raw mode\n", cfd);
+		gpsd_report(3, "client(%d) turned off raw mode\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",R=0");
 	    } else {
 		(void)assign_channel(whoami);
 		subscribers[cfd].raw = 1;
-		gpsd_report(3, "%d turned on raw mode\n", cfd);
+		gpsd_report(3, "client(%d) turned on raw mode\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",R=1");
 	    }
 	    break;
@@ -840,7 +887,7 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 	    } else {
 		subscribers[cfd].watcher = true;
 		(void)assign_channel(whoami);
-		gpsd_report(3, "%d turned on watching\n", cfd);
+		gpsd_report(3, "client(%d) turned on watching\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",W=1");
 	    }
 	    break;
@@ -900,17 +947,17 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 		p++;		
 	    } else if (*p == '1' || *p == '+') {
 		whoami->device->gpsdata.profiling = true;
-		gpsd_report(3, "%d turned on profiling mode\n", cfd);
+		gpsd_report(3, "client(%d) turned on profiling mode\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",Z=1");
 		p++;
 	    } else if (*p == '0' || *p == '-') {
 		whoami->device->gpsdata.profiling = false;
-		gpsd_report(3, "%d turned off profiling mode\n", cfd);
+		gpsd_report(3, "client(%d) turned off profiling mode\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",Z=0");
 		p++;
 	    } else {
 		whoami->device->gpsdata.profiling = !whoami->device->gpsdata.profiling;
-		gpsd_report(3, "%d toggled profiling mode\n", cfd);
+		gpsd_report(3, "client(%d) toggled profiling mode\n", cfd);
 		(void)snprintf(phrase, sizeof(phrase), ",Z=%d",
 			       (int)whoami->device->gpsdata.profiling);
 	    }
@@ -978,7 +1025,7 @@ static void handle_control(int sfd, char *buf)
 	    gpsd_report(1,"<= control(%d): %s already active \n", sfd, stash);
 	else {
 	    gpsd_report(1,"<= control(%d): adding %s \n", sfd, stash);
-	    if (open_device(stash, true))
+	    if (open_device(stash))
 		(void)write(sfd, "OK\n", 3);
 	    else
 		(void)write(sfd, "ERROR\n", 6);
@@ -1126,7 +1173,7 @@ int main(int argc, char *argv[])
 	rtcm_service = getservbyname("rtcm", "tcp") ? "rtcm" : DEFAULT_RTCM_PORT;
     /*@ +observertrans @*/
     if ((nsock = passivesock(rtcm_service, "tcp", QLEN)) < 0) {
-	gpsd_report(0,"rtcm socket create failed, netlib error %d\n",nsock);
+	gpsd_report(0,"RTCM104 socket create failed, netlib error %d\n",nsock);
 	exit(2);
     }
     gpsd_report(1, "listening on port %s\n", rtcm_service);
@@ -1217,7 +1264,7 @@ int main(int argc, char *argv[])
 	context.valid |= LEAP_SECOND_VALID;
 
     for (i = optind; i < argc; i++) { 
-	device = open_device(argv[i], nowait);
+	device = open_device(argv[i]);
 	if (!device) {
 	    gpsd_report(0, "GPS device %s nonexistent or can't be read\n", argv[i]);
 	}
@@ -1246,12 +1293,18 @@ int main(int argc, char *argv[])
 	{
 	    char dbuf[BUFSIZ];
 	    dbuf[0] = '\0';
+	    for (cfd = 0; cfd < FD_SETSIZE; cfd++)
+		if (FD_ISSET(cfd, &all_fds))
+		    (void)snprintf(dbuf + strlen(dbuf), 
+				   sizeof(dbuf)-strlen(dbuf),
+				   " %d", cfd);
+	    strcat(dbuf, "} -> {");
 	    for (cfd = 0; cfd < FD_SETSIZE; cfd++)
 		if (FD_ISSET(cfd, &rfds))
 		    (void)snprintf(dbuf + strlen(dbuf), 
 				   sizeof(dbuf)-strlen(dbuf),
 				   " %d", cfd);
-	    gpsd_report(4, "New input on these descriptors: %s\n", dbuf);
+	    gpsd_report(4, "Polling descriptor set: {%s}\n", dbuf);
 	}
 #endif /* UNUSED */
 
@@ -1271,9 +1324,7 @@ int main(int argc, char *argv[])
 		FD_SET(ssock, &all_fds);
 		subscribers[ssock].active = timestamp();
 		subscribers[ssock].tied = false;
-#ifdef RTCM104_SERVICE
-		subscribers[ssock].rtcm = false;
-#endif /* RTCM104_SERVICE */
+		subscribers[ssock].requires = ANY;
 	    }
 	    FD_CLR(msock, &rfds);
 	}
@@ -1295,7 +1346,7 @@ int main(int argc, char *argv[])
 		FD_SET(ssock, &all_fds);
 		subscribers[rsock].active = true;
 		subscribers[rsock].tied = false;
-		subscribers[rsock].rtcm = true;
+		subscribers[rsock].requires = RTCM104;
 	    }
 	    FD_CLR(nsock, &rfds);
 	}
@@ -1339,22 +1390,15 @@ int main(int argc, char *argv[])
 	    if (!allocated_channel(channel))
 		continue;
 
-	    /* we may need to force the GPS open */
-	    if (nowait && channel->gpsdata.gps_fd == -1) {
-		gpsd_deactivate(channel);
-		if (gpsd_activate(channel) >= 0) {
-		    FD_SET(channel->gpsdata.gps_fd, &all_fds);
-		    notify_watchers(channel, "GPSD,X=%f\r\n", timestamp());
-		}
-	    }
-
 	    /* pass the current DGPSIP correction to the GPS if new */
-	    dgpsip_relay(channel);
+	    if (channel->device_type)
+		dgpsip_relay(channel);
 
 	    /* get data from the device */
 	    changed = 0;
 	    if (channel->gpsdata.gps_fd >= 0 && FD_ISSET(channel->gpsdata.gps_fd, &rfds))
 	    {
+		gpsd_report(5, "polling %d\n", channel->gpsdata.gps_fd);
 		changed = gpsd_poll(channel);
 		if (changed == ERROR_SET) {
 		    gpsd_report(3, "packet sniffer failed to sync up\n");
@@ -1424,16 +1468,17 @@ int main(int argc, char *argv[])
 	for (cfd = 0; cfd < FD_SETSIZE; cfd++) {
 	    if (subscribers[cfd].active == 0) 
 		continue;
-	    else if (FD_ISSET(cfd, &rfds)) {
+
+	    if (FD_ISSET(cfd, &rfds)) {
 		char buf[BUFSIZ];
 		int buflen;
 
-		gpsd_report(3, "checking %d\n", cfd);
+		gpsd_report(3, "checking client(%d)\n", cfd);
 		if ((buflen = (int)read(cfd, buf, sizeof(buf) - 1)) <= 0) {
 		    detach_client(cfd);
 		} else {
 		    buf[buflen] = '\0';
-		    gpsd_report(1, "<= client: %s", buf);
+		    gpsd_report(1, "<= client(%d): %s", cfd, buf);
 
 #ifdef RTCM104_SERVICE
 		    if (subscribers[cfd].rtcm) {
@@ -1447,33 +1492,42 @@ int main(int argc, char *argv[])
 			if (handle_gpsd_request(cfd, buf, buflen) < 0)
 			    detach_client(cfd);
 		    }
-
-
 		}
 	    } else if (subscribers[cfd].device == NULL && timestamp() - subscribers[cfd].active > ASSIGNMENT_TIMEOUT) {
-		gpsd_report(1, "client timed out before assignment request.\n");
+		gpsd_report(1, "client(%d) timed out before assignment request.\n", cfd);
 		detach_client(cfd);
 	    } else if (subscribers[cfd].device != NULL && !(subscribers[cfd].watcher || subscribers[cfd].raw>0) && timestamp() - subscribers[cfd].active > POLLER_TIMEOUT) {
-		gpsd_report(1, "client timed out on command wait.\n");
+		gpsd_report(1, "client(%d) timed out on command wait.\n", cfd);
 		detach_client(cfd);
 	    }
 	}
 
-	/* close devices with no remaining subscribers */
-	for (channel = channels; channel < channels + MAXDEVICES; channel++) {
-	    if (allocated_channel(channel)) {
-		bool need_gps = false;
+	/*
+	 * Close devices with an identified packet type but no remaining 
+	 * subscribers.  The reason the test has this particular form is 
+	 * so that, immediately after device open, we'll keep reading 
+	 * packets until a type is identified even though there are no
+	 * subscribers yet.  We need this to happen so that subscribers 
+	 * can later choose a device by packet type.
+	 */
+	if (!nowait)
+	    for (channel=channels; channel < channels+MAXDEVICES; channel++) {
+		if (allocated_channel(channel)) {
+		    if (channel->packet_type != BAD_PACKET) {
+			bool need_gps = false;
 
-		for (cfd = 0; cfd < FD_SETSIZE; cfd++)
-		    if (subscribers[cfd].device == channel)
-			need_gps = true;
+			for (cfd = 0; cfd < FD_SETSIZE; cfd++)
+			    if (subscribers[cfd].device == channel)
+				need_gps = true;
 
-		if (!nowait && !need_gps && channel->gpsdata.gps_fd > -1) {
-		    FD_CLR(channel->gpsdata.gps_fd, &all_fds);
-		    gpsd_deactivate(channel);
+			if (!need_gps && channel->gpsdata.gps_fd > -1) {
+			    gpsd_report(4, "unflagging descriptor %d in open_device\n", channel->gpsdata.gps_fd);
+			    FD_CLR(channel->gpsdata.gps_fd, &all_fds);
+			    gpsd_deactivate(channel);
+			}
+		    }
 		}
 	    }
-	}
     }
 
     if (control_socket)
