@@ -4,6 +4,7 @@
 /* gpsd.h -- fundamental types and structures for the GPS daemon */
 
 #include <stdbool.h>
+#include <stdio.h>
 #include <termios.h>
 #include "config.h"
 #include "gps.h"
@@ -12,7 +13,7 @@
 #ifdef EARTHMATE_ENABLE
 #define ZODIAC_ENABLE	
 #endif
-#if defined(ZODIAC_ENABLE) || defined(SIRFII_ENABLE) || defined(GARMIN_ENABLE)
+#if defined(ZODIAC_ENABLE) || defined(SIRFII_ENABLE) || defined(GARMIN_ENABLE) || defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) || defined(ITALK_ENABLE)
 #define BINARY_ENABLE	
 #endif
 #if defined(TRIPMATE_ENABLE) || defined(BINARY_ENABLE)
@@ -22,23 +23,25 @@
 #define NMEA_MAX	82		/* max length of NMEA sentence */
 #define NMEA_BIG_BUF	(2*NMEA_MAX+1)	/* longer than longest NMEA sentence */
 
-/*  
- * From the RCTM104 standard:
- *
- * "The 30 bit words (as opposed to 32 bit words) coupled with a 50 Hz
- * transmission rate provides a convenient timing capability where the
- * times of word boundaries are a rational multiple of 0.6 seconds."
- *
- * "Each frame is N+2 words long, where N is the number of message data
- * words. For example, a filler message (type 6 or 34) with no message
- * data will have N=0, and will consist only of two header words. The
- * maximum number of data words allowed by the format is 31, so that
- * the longest possible message will have a total of 33 words."
- */
-#define RTCM_MAX	33*sizeof(int32_t)
+/* factors for converting among confidence interval units */
+#define CEP50_SIGMA	1.18
+#define DRMS_SIGMA	1.414
+#define CEP95_SIGMA	2.45
+
+/* this is where we choose the confidence level to use in reports */
+#define GPSD_CONFIDENCE	CEP95_SIGMA
+
+enum isgpsstat_t {
+    ISGPS_NO_SYNC, ISGPS_SYNC, ISGPS_SKIP, ISGPS_MESSAGE,
+};
+#define PREAMBLE_PATTERN 0x66
+
+#define ISGPS_ERRLEVEL_BASE	5
 
 #define NTPSHMSEGS	4		/* number of NTP SHM segments */
 
+#define RTCM_MAX	(RTCM_WORDS_MAX * sizeof(isgps30bits_t))
+
 struct gps_context_t {
     int valid;				/* member validity flags */
 #define LEAP_SECOND_VALID	0x01	/* we have or don't need correction */
@@ -67,6 +70,7 @@ struct gps_type_t {
 /* GPS method table, describes how to talk to a particular GPS type */
     /*@observer@*/char *typename;
     /*@observer@*//*@null@*/char *trigger;
+    int channels;
     /*@null@*/bool (*probe)(struct gps_device_t *session);
     /*@null@*/void (*initializer)(struct gps_device_t *session);
     /*@null@*/ssize_t (*get_packet)(struct gps_device_t *session);
@@ -74,8 +78,10 @@ struct gps_type_t {
     /*@null@*/ssize_t (*rtcm_writer)(struct gps_device_t *session, char *rtcmbuf, size_t rtcmbytes);
     /*@null@*/bool (*speed_switcher)(struct gps_device_t *session, speed_t speed);
     /*@null@*/void (*mode_switcher)(struct gps_device_t *session, int mode);
+    /*@null@*/bool (*rate_switcher)(struct gps_device_t *session, double rate);
+    int cycle_chars;
     /*@null@*/void (*wrapup)(struct gps_device_t *session);
-    int cycle;
+    double cycle;
 };
 
 #if defined (HAVE_SYS_TERMIOS_H)
@@ -95,6 +101,16 @@ struct gps_type_t {
  */
 #define MAX_PACKET_LENGTH	196	/* 188 + 8 */
 
+/*
+ * We used to define the inbut buffer length as MAX_PACKET_LENGTH*2+1.
+ * However, as it turns out, this isn't enough.  We've had a report
+ * from one user with a GPS that reports at 20Hz that "sometimes a
+ * long/slow context switch will cause the next read from the serial
+ * device to be very big. I sometimes see a read of 250 characters or
+ * more."
+ */
+#define INPUT_BUFFER_LENGTH	1024
+
 struct gps_device_t {
 /* session object, encapsulates all global state */
     struct gps_data_t gpsdata;
@@ -109,7 +125,11 @@ struct gps_device_t {
 #define SIRF_PACKET	1
 #define ZODIAC_PACKET	2
 #define TSIP_PACKET	3
+#define EVERMORE_PACKET	4
+#define ITALK_PACKET	5
+#define RTCM_PACKET	6
     unsigned int baudindex;
+    int saved_baud;
     unsigned int packet_state;
     size_t packet_length;
     unsigned char inbuffer[MAX_PACKET_LENGTH*2+1];
@@ -117,7 +137,8 @@ struct gps_device_t {
     unsigned /*@observer@*/char *inbufptr;
     unsigned char outbuffer[MAX_PACKET_LENGTH+1];
     size_t outbuflen;
-    unsigned long counter;
+    unsigned long char_counter;		/* count characters processed */
+    unsigned long retry_counter;	/* count sniff retries */
     double poll_times[FD_SETSIZE];	/* last daemon poll time */
 #ifdef NTPSHM_ENABLE
     int shmTime;
@@ -125,10 +146,23 @@ struct gps_device_t {
     int shmTimeP;
 # endif /* PPS_ENABLE */
 #endif /* NTPSHM_ENABLE */
-#ifdef BINARY_ENABLE
     struct gps_fix_t lastfix;	/* use to compute uncertainties */
     double mag_var;		/* Magnetic variation in degrees */  
+    /*
+     * The rest of this structure is driver-specific private storage.
+     * Because the Garmin driver uses a long buffer, you can have
+     * up to 4096+12 bytes of private storage in your own union member
+     * without making this structure larger or changing the API at all.
+     */
     union {
+#ifdef NMEA_ENABLE
+	struct {
+	    int part, await;		/* for tracking GSV parts */
+	    struct tm date;
+	    double subseconds;
+	} nmea;
+#endif /* NMEA_ENABLE */
+#ifdef BINARY_ENABLE
 #ifdef SIRFII_ENABLE
 	struct {
 	    unsigned int driverstate;	/* for private use */
@@ -153,6 +187,7 @@ struct gps_device_t {
 	    time_t last_41;		/* Timestamps for packet requests */
 	    time_t last_5c;
 	    time_t last_6d;
+	    unsigned int parity, stopbits; /* saved RS232 link parameters */
 	} tsip;
 #endif /* TSIP_ENABLE */
 #ifdef GARMIN_ENABLE	/* private housekeeping stuff for the Garmin driver */
@@ -174,8 +209,21 @@ struct gps_device_t {
 	    unsigned int Zv[ZODIAC_CHANNELS];	/* signal values (0-7) */
 	} zodiac;
 #endif /* ZODIAC_ENABLE */
-    };
+	/*
+	 * This is not conditionalized on RTCM104_ENABLE because we need to
+	 * be able to build rtcmdecode even when RTCM support is not
+	 * configured in the daemon.  It doesn't take up extra space.
+	 */
+	struct {
+	    /* ISGPS200 decodng */
+	    bool            locked;
+	    int             curr_offset;
+	    isgps30bits_t   curr_word;
+	    isgps30bits_t   buf[RTCM_WORDS_MAX];
+	    unsigned int    bufindex;
+	} isgps;
 #endif /* BINARY_ENABLE */
+    } driver;
 };
 
 #define IS_HIGHEST_BIT(v,m)	(v & ~((m<<1)-1))==0
@@ -184,14 +232,18 @@ struct gps_device_t {
 extern struct gps_type_t **gpsd_drivers;
 
 /* GPS library internal prototypes */
-extern gps_mask_t nmea_parse(char *, struct gps_data_t *);
+extern gps_mask_t nmea_parse_input(struct gps_device_t *);
+extern gps_mask_t nmea_parse(char *, struct gps_device_t *);
 extern int nmea_send(int, const char *, ... );
 extern void nmea_add_checksum(char *);
 
-extern gps_mask_t sirf_parse(struct gps_device_t *, unsigned char *, size_t);
+ssize_t pass_rtcm(struct gps_device_t *, char *, size_t);
 
+extern gps_mask_t sirf_parse(struct gps_device_t *, unsigned char *, size_t);
+extern gps_mask_t evermore_parse(struct gps_device_t *, unsigned char *, size_t);
 extern void packet_reset(struct gps_device_t *);
 extern void packet_pushback(struct gps_device_t *);
+extern ssize_t packet_parse(struct gps_device_t *, size_t);
 extern ssize_t packet_get(struct gps_device_t *);
 extern int packet_sniff(struct gps_device_t *);
 
@@ -203,6 +255,7 @@ extern void dgpsip_autoconnect(struct gps_context_t *,
 			       double, double, const char *);
 
 extern int gpsd_open(struct gps_device_t *);
+extern bool gpsd_write(struct gps_device_t *, void const *, size_t);
 extern bool gpsd_next_hunt_setting(struct gps_device_t *);
 extern int gpsd_switch_driver(struct gps_device_t *, char *);
 extern void gpsd_set_speed(struct gps_device_t *, speed_t, unsigned char, unsigned int);
@@ -210,22 +263,48 @@ extern speed_t gpsd_get_speed(struct termios *);
 extern void gpsd_close(struct gps_device_t *);
 
 extern void gpsd_zero_satellites(/*@out@*/struct gps_data_t *sp)/*@modifies sp@*/;
+extern void gpsd_interpret_subframe(struct gps_device_t *, unsigned int[]);
+extern /*@ observer @*/ char *gpsd_hexdump(const void *, size_t);
 extern int netlib_connectsock(const char *, const char *, const char *);
 
 extern int ntpshm_init(struct gps_context_t *, bool);
-extern int ntpshm_alloc(struct gps_context_t *context);
-extern bool ntpshm_free(struct gps_context_t *context, int segment);
+extern int ntpshm_alloc(struct gps_context_t *);
+extern bool ntpshm_free(struct gps_context_t *, int);
 extern int ntpshm_put(struct gps_device_t *, double);
 extern int ntpshm_pps(struct gps_device_t *,struct timeval *);
 
+extern void isgps_init(/*@out@*/struct gps_device_t *);
+enum isgpsstat_t isgps_decode(struct gps_device_t *, 
+			      bool (*preamble_match)(isgps30bits_t *),
+			      bool (*length_check)(struct gps_device_t *),
+			      size_t,
+			      unsigned int);
+extern unsigned int isgps_parity(isgps30bits_t);
+extern enum isgpsstat_t rtcm_decode(struct gps_device_t *, unsigned int);
+
+extern void rtcm_dump(struct gps_device_t *, /*@out@*/char[], size_t);
+extern int rtcm_undump(/*@out@*/struct rtcm_t *, char *);
+extern void rtcm_unpack(struct gps_device_t *);
+extern bool rtcm_repack(struct gps_device_t *);
+
 extern void ecef_to_wgs84fix(struct gps_data_t *,
 			     double, double, double, 
 			     double, double, double);
 extern gps_mask_t dop(struct gps_data_t *);
 
-/* External interface */
-extern void gpsd_init(struct gps_device_t *, 
-			     struct gps_context_t *, char *device);
+/* srecord.c */
+extern void hexdump(size_t, unsigned char *, unsigned char *);
+extern unsigned char sr_sum(unsigned int, unsigned int, unsigned char *);
+extern int bin2srec(unsigned int, unsigned int, unsigned int, unsigned char *, unsigned char *);
+extern int srec_hdr(unsigned int, unsigned char *, unsigned char *);
+extern int srec_fin(unsigned int, unsigned char *);
+extern unsigned char hc(unsigned char);
+
+/* exported bits for the GPS flasher */
+bool sirf_write(int fd, unsigned char *msg);
+
+/* application interface */
+extern void gpsd_init(struct gps_device_t *, struct gps_context_t *, char *);
 extern int gpsd_activate(struct gps_device_t *);
 extern void gpsd_deactivate(struct gps_device_t *);
 extern gps_mask_t gpsd_poll(struct gps_device_t *);
@@ -238,14 +317,24 @@ void gpsd_report(int, const char *, ...);
 extern bool finite(double);
 extern int cfmakeraw(struct termios *);
 extern struct protoent *getprotobyname(const char *);
-extern /*@observer@*/char *strptime(const char *,const char *,/*@out@*/struct tm *tp)/*@modifies tp@*/;
+extern /*@observer@*/char *strptime(const char *,const char *tp,/*@out@*/struct tm *)/*@modifies tp@*/;
 extern struct tm *gmtime_r(const time_t *,/*@out@*/struct tm *tp)/*@modifies tp@*/;
 extern struct tm *localtime_r(const time_t *,/*@out@*/struct tm *tp)/*@modifies tp@*/;
 extern float roundf(float x);
 #endif /* S_SPLINT_S */
 
-/* BSD port hack */ 
-#define rint(x)	round(x)
-#define rintf(x) roundf(x)
+/* some OSes don't have round(). fake it if need be */
+#ifndef HAVE_ROUND
+#define	round(x) ((double)rint(x))
+#define roundf(x) ((float)rintf(x))
+#endif /* !HAVE_ROUND */
+
+/* OpenBSD and FreeBSD don't seem to have NAN, NetBSD does, others? */
+/* XXX test for this in configure? */
+#if defined(__OpenBSD__) || defined(__FreeBSD__)
+#ifndef NAN
+#define NAN (0.0/0.0)
+#endif /* !NAN */
+#endif /* list of Operating Systems */
 
 #endif /* _gpsd_h_ */
