@@ -268,6 +268,8 @@ class DaemonInstance:
             return False
     def fd_set(self):
         "Return the set of file descriptors currently opened by the daemon."
+        if self.pid == None:
+            return []
         fds = map(int, os.listdir("/proc/%d/fd" % self.pid))
         # I wish I knew what the entries above 1000 in Linux /proc/*/fd mean...
         return filter(lambda x: x < 1000, fds)
@@ -333,6 +335,7 @@ class TestSession:
                 newgps.go_predicate = self.default_predicate
             self.fakegpslist[newgps.slave] = newgps
         self.daemon.add_device(newgps.slave)
+        self.fakegpslist[newgps.slave].start(thread=True)
         self.sanity_check()
         return newgps.slave
     def gps_remove(self, name):
@@ -345,23 +348,15 @@ class TestSession:
         "Initiate a client session and force connection to a fake GPS."
         self.progress("gpsfake: client_add()\n")
         newclient = gps.gps()
-        newclient.id = self.client_id+1 
         self.clients.append(newclient)
-        newclient.query("of\n")
-        time.sleep(1)	# Avoid mysterious "connection reset by peer"
-        if not newclient.device:
-            self.progress("gpsd: returned no device for client open.\n")
-            self.sanity_check()
-            return None
-        else:
-            self.client_id += 1
-            self.progress("gpsfake: client %d has %s\n" % (self.client_id,newclient.device))
-            self.fakegpslist[newclient.device].start(thread=True)
-            newclient.set_thread_hook(lambda x: self.reporter(x))
-            if commands:
-                newclient.query(commands)
-            self.sanity_check()
-            return newclient.id
+        newclient.id = self.client_id + 1 
+        self.client_id += 1
+        self.progress("gpsfake: client %d has %s\n" % (self.client_id,newclient.device))
+        newclient.set_thread_hook(lambda x: self.reporter(x))
+        if commands:
+            newclient.query(commands)
+        self.sanity_check()
+        return newclient.id
     def client_query(self, id, commands):
         "Ship a command down a client channel, accept a response."
         self.progress("gpsfake: client_query(%d, %s)\n" % (id, `commands`))
