@@ -612,6 +612,7 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 			  (sizeof(buf2)-strlen(buf2)));
 	    else
 #endif /* RTCM104_ENABLE */
+#ifdef BINARY_ENABLE
 	    if ((session->gpsdata.set & LATLON_SET) != 0)
 		gpsd_binary_fix_dump(session, 
 				     buf2+strlen(buf2), 
@@ -624,6 +625,7 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 		gpsd_binary_satellite_dump(session,
 					 buf2 + strlen(buf2),
 					 (sizeof(buf2)-strlen(buf2)));
+#endif /* BINARY_ENABLE */
 	    if (buf2[0] != '\0') {
 		gpsd_report(3, "<= GPS: %s", buf2);
 		if (session->gpsdata.raw_hook)
@@ -658,7 +660,7 @@ char /*@ observer @*/ *gpsd_hexdump(const void *binbuf, size_t binbuflen)
     static char hexbuf[MAX_PACKET_LENGTH*2+1];
     size_t i;
     size_t len = (size_t)((binbuflen > MAX_PACKET_LENGTH) ? MAX_PACKET_LENGTH : binbuflen);
-    char *ibuf = (char *)binbuf;
+    const char *ibuf = (const char *)binbuf;
     memset(hexbuf, 0, sizeof(hexbuf));
 
     for (i = 0; i < len; i++) {
