@@ -794,9 +794,15 @@ ssize_t packet_get(struct gps_device_t *session)
     newdata = read(session->gpsdata.gps_fd, session->inbuffer+session->inbuflen,
 			sizeof(session->inbuffer)-(session->inbuflen));
     /*@ +modobserver @*/
-    if (newdata < 0 && errno != EAGAIN)
-	return BAD_PACKET;
-    else if (newdata == 0 || (newdata < 0 && errno == EAGAIN))
+    if (newdata == -1){ /*newdata == -1 && errno = (EAGAIN || EINTR) */
+        if ((errno == EAGAIN) || (errno == EINTR)) {
+	    return 0;
+        } else {
+	    return BAD_PACKET;
+        }
+    }
+
+    if (newdata == 0)
 	return 0;
     return packet_parse(session, (size_t)newdata);
 }
@@ -824,6 +830,7 @@ void packet_pushback(struct gps_device_t *session)
 		session->outbuffer,
 		session->outbuflen);
 	session->inbuflen += session->outbuflen;
+	session->inbufptr += session->outbuflen;
 	session->outbuflen = 0;
     }
 }
