@@ -102,6 +102,12 @@ int gpsd_open(struct gps_device_t *session)
 	return -1;
     }
 
+    if (session->saved_baud != -1) {
+        /*@i@*/(void)cfsetspeed(&session->ttyset, (speed_t)session->saved_baud);
+	(void)tcsetattr(session->gpsdata.gps_fd, TCSANOW, &session->ttyset);
+	(void)tcflush(session->gpsdata.gps_fd, TCIOFLUSH);
+    }
+
     session->packet_type = BAD_PACKET;
     if (isatty(session->gpsdata.gps_fd)!=0) {
 #ifdef NON_NMEA_ENABLE
@@ -142,20 +148,33 @@ int gpsd_open(struct gps_device_t *session)
 	session->ttyset.c_cflag |= CREAD | CLOCAL;
 	session->ttyset.c_iflag = session->ttyset.c_oflag = session->ttyset.c_lflag = (tcflag_t) 0;
 
-	session->counter = session->baudindex = 0;
+	session->baudindex = 0;
 	gpsd_set_speed(session, 
 		       gpsd_get_speed(&session->ttyset_old), 'N', 1);
     }
     return session->gpsdata.gps_fd;
 }
 
+bool gpsd_write(struct gps_device_t *session, void const *buf, size_t len)
+{
+     ssize_t status;
+     bool ok;
+     status = write(session->gpsdata.gps_fd, buf, len);	
+     ok = (status == (ssize_t)len);
+     (void)tcdrain(session->gpsdata.gps_fd);
+     /* code that will check for data could be add here to print buffer as text or hex */
+     /* no test here now, always print as hex */
+     gpsd_report(5, "=> GPS: %s%s\n", gpsd_hexdump(buf, len), ok?"":" FAILED");
+     return ok;
+}
+
 /*
  * This constant controls how long the packet sniffer will spend looking
  * for a packet leader before it gives up.  It *must* be larger than
  * MAX_PACKET_LENGTH or we risk never syncing up at all.  Large values
  * will produce annoying startup lag.
  */
-#define SNIFF_RETRIES	600
+#define SNIFF_RETRIES	256
 
 bool gpsd_next_hunt_setting(struct gps_device_t *session)
 /* advance to the next hunt setting  */
@@ -163,8 +182,8 @@ bool gpsd_next_hunt_setting(struct gps_device_t *session)
     /* every rate we're likely to see on a GPS */
     static unsigned int rates[] = {0, 4800, 9600, 19200, 38400, 57600};
 
-    if (session->counter++ >= SNIFF_RETRIES) {
-	session->counter = 0;
+    if (session->retry_counter++ >= SNIFF_RETRIES) {
+	session->retry_counter = 0;
 	if (session->baudindex++ >= (unsigned int)(sizeof(rates)/sizeof(rates[0]))) {
 	    session->baudindex = 0;
 	    if (session->gpsdata.stopbits++ >= 2)
